/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "E:\SDK2_SE_R\SEMod_struct.h"
#include "E:\SDK2_SE_R\SEPin.h"
#include <assert.h>
#include <string.h>
#include <stdio.h> // files
#include <io.h> // filelenght ..

char buffer[2000];
char name[100];

void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
	// system messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;

	if( !is_system_msg )
	{
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		int chan = midi_msg & 0x0f;
		int stat = midi_msg & 0xf0;// the next two bytes depend on the type of message
		int b2 = ( midi_msg / 0x100 ) & 0xff;
		int b3 = ( midi_msg / 0x10000 ) & 0xff;
       
		    if ((stat == NOTE_ON)||(stat==NOTE_OFF)) b2 += bmid + transpose_ammount ; // byte 2 is the note number 
			if (((stat == NOTE_ON)||(stat==NOTE_OFF))&& ( b2 <= 127 ) && ( b2 >= 0 ))
			{
			 midi_msg = stat + chan + (b2 << 8) + (b3 << 16); // re-assemble the MIDI message into an 32 bit integer
             getPin(PN_MIDI_OUT)->TransmitMIDI( p_clock, midi_msg );// send the MIDI
			}
	}
}


void Module::sub_process(long buffer_offset, long sampleFrames )
{
	float *in = buffer_offset + input1_buffer;
	
    int pol;
	while(--sampleFrames >= 0)
	{
           shift = *in;
           if   (shift == 0.0f)
		   {
		        pol = 0;
		   } 
		   else  
		   {  
		    if (shift > 0.0f )
			{			
				pol = -1; 
			}
			else
			{ 
		        pol = +1;  
			}
		   }
		bmid = (int) ( -pol + (shift + (pol * 0.004)) / 0.008333333333333333);  // +10..-10V = +1.0f..-1.0f = delta 2.0  * 64 =  0...127 bmid (note out midinr) 
		in++;
	}
}


Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);



}

Module::~Module()
{
	// nothing to see here, move along.
}

// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	
int j;

sprintf_s( name    ,100, "%s\n", " ***** ");

//sprintf_s( dish.thisSEM ,100, "%s\n", "+++++++++++++++++++++++++++" );
  

j  = sprintf_s( buffer    ,2000, "%s\n", " *********************************************" );
j  = sprintf_s( buffer + j,2000, "%s\n", " *   2010 Mai in Tbingen GERMANY" );
j += sprintf_s( buffer + j,2000, "%s\n", " *            FAKE MUSIC" ); 
j += sprintf_s( buffer + j,2000, "%s\n", " *        by ralph.a.cronin" ); 
j += sprintf_s( buffer + j,2000, "%s\n", " * made while presearch a room at" );
j += sprintf_s( buffer + j,2000, "%s\n", " *   Kln Germany over internet" ); 
j += sprintf_s( buffer + j,2000, "%s\n", " *********************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " ***** ");
j += sprintf_s( buffer + j,2000, "%s\n", "--------------------------------------------------------------------");
j += sprintf_s( buffer + j,2000, "%s\n", " *****************************************************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " * The IN_PITCH input shifts the midi notes also this way" );
j += sprintf_s( buffer + j,2000, "%s\n", " * but takes an audio input signal. ( Wave , Slider, ...)" ); 
j += sprintf_s( buffer + j,2000, "%s\n", " * notes are only transmitted from range (0...127 Overs are supressed) " );
j += sprintf_s( buffer + j,2000, "%s\n", " *****************************************************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " *     to the given income midi note_on and note_off        " );
j += sprintf_s( buffer + j,2000, "%s\n", " *     ##### 1 Volt makes offset of one Oktave #####.       " );
j += sprintf_s( buffer + j,2000, "%s\n", " *     0,0833 Volt is a halfnote. 12 Halfnotes is a oktave. " );
j += sprintf_s( buffer + j,2000, "%s\n", " *     Try things like Okave higher/deeper (+12)(-12)       " );
j += sprintf_s( buffer + j,2000, "%s\n", " *     Quint higher (+7.0), Quart higher (+5.0)             " );
j += sprintf_s( buffer + j,2000, "%s\n", " ******************************************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " ************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " **          enjoy SYNTHEDIT          " );
j += sprintf_s( buffer + j,2000, "%s\n", " **        by Jeff Mc Clintock        " );
j += sprintf_s( buffer + j,2000, "%s\n", " ************************************" );
j += sprintf_s( buffer + j,2000, "%s\n", " next planed Modul is MIDI_SHIFT_CT.sem : " );
j += sprintf_s( buffer + j,2000, "%s\n", " with control timedelay of NOTE_OFF at note pitch changed." );  
    

    properties->name  = " ***** ";
    properties->id    = " ***** ";
	properties->about = buffer;//"by ralph a. cronin and Jeff M";
	return true;
}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
    case 1:
		properties->name				= "IN_SHIFT";
        properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype_extra      = "5,-5,5,-5";
		properties->datatype			= DT_FSAMPLE;
		break;
	case 2:
		properties->name				= "MIDI Out";
		properties->direction			= DR_OUT;
		properties->datatype			= DT_MIDI2;
		break;
	
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}


void Module::OnPlugStateChange(SEPin *pin)
{ 
	
	
	float ist = getPin(PN_MIDIPITCH)->getValue();
	if(inn == ist)
	{
       getPin(PN_MIDIPITCH)->TransmitStatusChange( SampleClock(), ST_STATIC);// statt out_stat ralph
	}
	else
	{
	 getPin(PN_MIDIPITCH)->TransmitStatusChange( SampleClock(), ST_RUN);
	 inn =  ist;
	}
	
}